@@ -73,21 +73,19 @@ SIMPLY_BLOCK_CLI_NAME = get_from_env_var_file(
         "SIMPLY_BLOCK_COMMAND_NAME", "sbcli")
 TASK_EXEC_INTERVAL_SEC = 10
 TASK_EXEC_RETRY_COUNT = 8
+SIMPLY_BLOCK_SPDK_CORE_IMAGE = "simplyblock/spdk-core:v24.05-tag-latest"
 
 
-SIMPLY_BLOCK_SPDK_ULTRA_IMAGE = "simplyblock/spdk:lvolstore-ha-snapshot-latest"
+SIMPLY_BLOCK_SPDK_ULTRA_IMAGE = "public.ecr.aws/simply-block/ultra:main-latest"
 
 GELF_PORT = 12202
 
 MIN_HUGE_PAGE_MEMORY_FOR_LVOL = 209715200
 MIN_SYS_MEMORY_FOR_LVOL = 524288000
 EXTRA_SMALL_POOL_COUNT = 4096
-EXTRA_LARGE_POOL_COUNT = 16000
-# EXTRA_SMALL_POOL_COUNT = 0
-# EXTRA_LARGE_POOL_COUNT = 0
-EXTRA_HUGE_PAGE_MEMORY = 2147483648
-#EXTRA_SYS_MEMORY = 2147483648
-EXTRA_SYS_MEMORY = 0
+EXTRA_LARGE_POOL_COUNT = 10240
+EXTRA_HUGE_PAGE_MEMORY = 1147483648
+EXTRA_SYS_MEMORY = 0.10
 
 INSTANCE_STORAGE_DATA = {
         'i4i.large': {'number_of_devices': 1, 'size_per_device_gb': 468},
@@ -127,4 +125,5 @@ LVOL_NVME_CONNECT_NR_IO_QUEUES=6
 QPAIR_COUNT=32
 NVME_TIMEOUT_US=20000000
 
-NVMF_MAX_SUBSYSTEMS=50000
\ No newline at end of file
+NVMF_MAX_SUBSYSTEMS=50000
+HA_JM_COUNT=3
