@@ -2,6 +2,7 @@
 
 from simplyblock_core import utils, distr_controller
 from simplyblock_core.db_controller import DBController
+from simplyblock_core.models.cluster import Cluster
 from simplyblock_core.models.nvme_device import NVMeDevice, JMDevice
 from simplyblock_core.models.storage_node import StorageNode
 from simplyblock_core.rpc_client import RPCClient
@@ -115,6 +116,10 @@ def _check_node_lvstore(lvstore_stack, node, auto_fix=False):
     logger.info(f"Checking distr stack on node : {node.get_id()}")
     rpc_client = RPCClient(
         node.mgmt_ip, node.rpc_port, node.rpc_username, node.rpc_password, timeout=3, retry=1)
+    cluster = db_controller.get_cluster_by_id(node.cluster_id)
+    if cluster.status in [Cluster.STATUS_INACTIVE, Cluster.STATUS_IN_ACTIVATION]:
+        auto_fix = False
+
     distribs_list = []
     raid = None
     bdev_lvstore = None
@@ -396,6 +401,9 @@ def check_lvol_on_node(lvol_id, node_id):
         return False
 
     snode = db_controller.get_storage_node_by_id(node_id)
+    if not snode:
+        return False
+
     rpc_client = RPCClient(
         snode.mgmt_ip, snode.rpc_port,
         snode.rpc_username, snode.rpc_password, timeout=5, retry=1)
@@ -443,7 +451,7 @@ def check_lvol(lvol_id):
         passed = True
         for nodes_id in lvol.nodes:
             node = db_controller.get_storage_node_by_id(nodes_id)
-            if node.status == StorageNode.STATUS_ONLINE:
+            if node and node.status == StorageNode.STATUS_ONLINE:
                 ret = check_lvol_on_node(lvol_id, nodes_id)
                 if not ret:
                     passed = False
