@@ -10,7 +10,8 @@ from datetime import datetime
 from typing import Tuple
 
 from simplyblock_core import utils, constants, distr_controller
-from simplyblock_core.controllers import snapshot_controller, pool_controller, lvol_events, caching_node_controller
+from simplyblock_core.controllers import snapshot_controller, pool_controller, lvol_events, caching_node_controller, \
+    tasks_controller
 from simplyblock_core.db_controller import DBController
 from simplyblock_core.models.nvme_device import NVMeDevice
 from simplyblock_core.models.pool import Pool
@@ -123,7 +124,7 @@ def validate_add_lvol_func(name, size, host_id_or_name, pool_id_or_name,
         total = pool_controller.get_pool_total_capacity(pool.get_id())
         if total + size > pool.pool_max_size:
             return False, f"Invalid LVol size: {utils.humanbytes(size)} " \
-                          f"Pool max size has reached {utils.humanbytes(total)} of {utils.humanbytes(pool.pool_max_size)}"
+                          f"Pool max size has reached {utils.humanbytes(total+size)} of {utils.humanbytes(pool.pool_max_size)}"
 
     for lvol in db_controller.get_lvols(pool.cluster_id):
         if lvol.pool_uuid == pool.get_id():
@@ -175,6 +176,11 @@ def _get_next_3_nodes(cluster_id, lvol_size=0):
             continue
 
         if node.status == node.STATUS_ONLINE:
+
+            lvol_count = len(db_controller.get_lvols_by_node_id(node.get_id()))
+            if lvol_count >= node.max_lvol:
+                continue
+
             # Validate Eligible nodes for adding lvol
             # snode_api = SNodeClient(node.api_endpoint)
             # result, _ = snode_api.info()
@@ -187,11 +193,10 @@ def _get_next_3_nodes(cluster_id, lvol_size=0):
             #     continue
             #
             online_nodes.append(node)
-            lvols = db_controller.get_lvols_by_node_id(node.get_id()) or []
             # node_stat_list = db_controller.get_node_stats(node, limit=1000)
             # combined_record = utils.sum_records(node_stat_list)
             node_st = {
-                "lvol": len(lvols)+1,
+                "lvol": lvol_count+1,
                 # "cpu": 1 + (node.cpu * node.cpu_hz),
                 # "r_io": combined_record.read_io_ps,
                 # "w_io": combined_record.write_io_ps,
@@ -319,6 +324,18 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     if cl.status not in [cl.STATUS_ACTIVE, cl.STATUS_DEGRADED]:
         return False, f"Cluster is not active, status: {cl.status}"
 
+    if uid:
+        for lvol in db_controller.get_lvols():
+            if lvol.get_id() == uid:
+                if pvc_name:
+                    lvol.pvc_name = pvc_name
+                if name:
+                    lvol.lvol_name = name
+                if namespace:
+                    lvol.namespace = namespace
+                lvol.write_to_db()
+                return uid, None
+
     if ha_type == "default":
         ha_type = cl.ha_type
 
@@ -380,7 +397,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     elif cl.prov_cap_warn and cl.prov_cap_warn < cluster_size_prov_util:
         logger.warning(f"Cluster provisioned cap warning, util: {cluster_size_prov_util}% of cluster util: {cl.prov_cap_warn}")
 
-    if distr_vuid == 0:
+    if not distr_vuid:
         vuid = utils.get_random_vuid()
     else:
         vuid = distr_vuid
@@ -402,7 +419,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     lvol.namespace = namespace or ""
     lvol.size = int(size)
     lvol.max_size = int(max_size)
-    lvol.status = LVol.STATUS_ONLINE
+    lvol.status = LVol.STATUS_IN_CREATION
 
     lvol.create_dt = str(datetime.now())
     lvol.ha_type = ha_type
@@ -473,57 +490,36 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
         lvol.crypto_key1 = crypto_key1
         lvol.crypto_key2 = crypto_key2
 
-    if ha_type == 'single':
-        ret, error = add_lvol_on_node(lvol, host_node)
-        if error:
-            return ret, error
-        lvol.nodes = [host_node.get_id()]
-    elif ha_type == "ha":
+    lvol.write_to_db(db_controller.kv_store)
 
-        lvol_bdev, error = add_lvol_on_node(lvol, host_node)
-        if error:
-            return False, error
-
-        """
-    "driver_specific": {
-      "lvol": {
-        "lvol_store_uuid": "0aa735b5-67d2-477a-95d4-4e7108f0e769",
-        "base_bdev": "raid0_9768",
-        "thin_provision": true,
-        "num_allocated_clusters": 0,
-        "snapshot": false,
-        "clone": false,
-        "lvol_leadership": true,
-        "lvs_leadership": true,
-        "blobid": 4294967297,
-        "lvol_priority_class": 0,
-        "esnap_clone": false
-      }
-    }
-        """
-
-        lvol.lvol_uuid = lvol_bdev['uuid']
-        lvol.blobid = lvol_bdev['driver_specific']['lvol']['blobid']
+    lvol_bdev, error = add_lvol_on_node(lvol, host_node)
+    if error:
+        return False, error
+    lvol.nodes = [host_node.get_id()]
+    lvol.lvol_uuid = lvol_bdev['uuid']
+    lvol.blobid = lvol_bdev['driver_specific']['lvol']['blobid']
 
+    if ha_type == "ha":
         sec_node = db_controller.get_storage_node_by_id(host_node.secondary_node_id)
-        ret, error = add_lvol_on_node(lvol, sec_node, ha_inode_self=1)
-        if error:
-            return ret, error
+        if sec_node and sec_node.status == StorageNode.STATUS_ONLINE:
+            ret, error = add_lvol_on_node(lvol, sec_node, ha_inode_self=1)
+            if error:
+                logger.error(error)
+                logger.error(f"Failed to add lvol on node: {sec_node.get_id()}")
+                logger.info(f"Removing LVol from {host_node.get_id()}")
+                lvol.status = LVol.STATUS_IN_DELETION
+                lvol.write_to_db(db_controller.kv_store)
+                ret=delete_lvol_from_node(lvol.get_id(), host_node.get_id())
+                if ret:
+                    lvol.remove(db_controller.kv_store)
+                else:
+                    logger.error(f"Failed to remove lvol from node {host_node.get_id()}, LVol status: {lvol.status}")
+                return False, error
 
-        nodes_ids = [
-            host_node.get_id(),
-            host_node.secondary_node_id]
-        lvol.nodes = nodes_ids
+            lvol.nodes.append(host_node.secondary_node_id)
 
-    # host_node = db_controller.get_storage_node_by_id(host_node.get_id())
-    # host_node.lvols += 1
-    # host_node.write_to_db(db_controller.kv_store)
-
-    # pool = db_controller.get_pool_by_id(pool.get_id())
     lvol.pool_uuid = pool.get_id()
-    # pool.lvols += 1
-    # pool.write_to_db(db_controller.kv_store)
-
+    lvol.status = LVol.STATUS_ONLINE
     lvol.write_to_db(db_controller.kv_store)
     lvol_events.lvol_create(lvol)
 
@@ -587,24 +583,17 @@ def _create_bdev_stack(lvol, snode):
 
 def add_lvol_on_node(lvol, snode, ha_comm_addrs=None, ha_inode_self=0):
     rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
-    spdk_mem_info_before = rpc_client.ultra21_util_get_malloc_stats()
 
     if snode.is_secondary_node:
         rpc_client.bdev_lvol_set_leader(False, lvs_name=lvol.lvs_name)
 
     else:
-        # Validate adding lvol on storage node
-        snode_api = SNodeClient(snode.api_endpoint)
-        result, _ = snode_api.info()
-        memory_free = result["memory_details"]["free"]
-        huge_free = result["memory_details"]["huge_free"]
 
-        total_node_capacity = db_controller.get_snode_size(snode.get_id())
-        lvols = len(db_controller.get_lvols_by_node_id(snode.get_id()))
-        error = utils.validate_add_lvol_or_snap_on_node(memory_free, huge_free, snode.max_lvol, lvol.size,  total_node_capacity, lvols)
-        if error:
+        lvol_count = len(db_controller.get_lvols_by_node_id(snode.get_id()))
+        if lvol_count >= snode.max_lvol:
+            error = f"Too many lvols on node: {snode.get_id()}, max lvols reached: {lvol_count}"
             logger.error(error)
-            return False, f"Failed to add lvol on node {snode.get_id()}"
+            return False, error
 
         rpc_client.bdev_lvol_set_leader(True, lvs_name=lvol.lvs_name)
 
@@ -638,23 +627,10 @@ def add_lvol_on_node(lvol, snode, ha_comm_addrs=None, ha_inode_self=0):
     if not ret:
         return False, "Failed to add bdev to subsystem"
 
-    # logger.info("Sending cluster map to LVol")
-    # ret = distr_controller.send_cluster_map_to_node(snode)
-    # if not ret:
-    #     return False, "Failed to send cluster map"
-
     spdk_mem_info_after = rpc_client.ultra21_util_get_malloc_stats()
     logger.debug("ultra21_util_get_malloc_stats:")
     logger.debug(spdk_mem_info_after)
 
-    diff = {}
-    for key in spdk_mem_info_after.keys():
-        diff[key] = spdk_mem_info_after[key] - spdk_mem_info_before[key]
-
-    logger.info("spdk mem diff:")
-    logger.info(json.dumps(diff, indent=2))
-    lvol.mem_diff = diff
-
     ret = rpc_client.get_bdevs(f"{lvol.lvs_name}/{lvol.lvol_bdev}")
     lvol_bdev = ret[0]
 
@@ -856,16 +832,16 @@ def delete_lvol(id_or_name, force_delete=False):
         snode.rpc_username,
         snode.rpc_password)
 
-    # soft delete LVol if it has snapshots
-    snaps = db_controller.get_snapshots()
-    for snap in snaps:
-        if snap.deleted is False and snap.lvol.get_id() == lvol.get_id():
-            logger.warning(f"Soft delete LVol that has snapshots. Snapshot:{snap.get_id()}")
-            ret = rpc_client.subsystem_delete(lvol.nqn)
-            logger.debug(ret)
-            lvol.deleted = True
-            lvol.write_to_db(db_controller.kv_store)
-            return True
+    # # soft delete LVol if it has snapshots
+    # snaps = db_controller.get_snapshots()
+    # for snap in snaps:
+    #     if snap.deleted is False and snap.lvol.get_id() == lvol.get_id():
+    #         logger.warning(f"Soft delete LVol that has snapshots. Snapshot:{snap.get_id()}")
+    #         ret = rpc_client.subsystem_delete(lvol.nqn)
+    #         logger.debug(ret)
+    #         lvol.deleted = True
+    #         lvol.write_to_db(db_controller.kv_store)
+    #         return True
 
     if snode.status not in [StorageNode.STATUS_ONLINE, StorageNode.STATUS_REMOVED]:
         logger.error(f"Node status is not online or removed, node: {snode.get_id()}, status: {snode.status}")
@@ -943,18 +919,6 @@ def delete_lvol(id_or_name, force_delete=False):
             if not force_delete:
                 return False
 
-    # remove from db
-    # snode = db_controller.get_storage_node_by_id(lvol.node_id)
-    # # logger.debug(snode)
-    # logger.debug(f"removing lvol: {lvol.get_id()} from node {snode.get_id()}")
-    # snode.lvols -= 1
-    # snode.write_to_db(db_controller.kv_store)
-
-    # # remove from pool
-    # pool = db_controller.get_pool_by_id(lvol.pool_uuid)
-    # pool.lvols -= 1
-    # pool.write_to_db(db_controller.kv_store)
-
     lvol_events.lvol_delete(lvol)
     lvol.remove(db_controller.kv_store)
 
@@ -1049,7 +1013,9 @@ def list_lvols(is_json, cluster_id, pool_id_or_name, all=False):
             "Size": utils.humanbytes(lvol.size),
             "Hostname": lvol.hostname,
             "HA": lvol.ha_type,
-            "Priority": lvol.lvol_priority_class,
+            "BlobID": lvol.blobid or "",
+            "LVolUUID": lvol.lvol_uuid or "",
+            # "Priority": lvol.lvol_priority_class,
             "Status": lvol.status,
             "IO Err": lvol.io_error,
             "Health": lvol.health_check,
@@ -1154,6 +1120,8 @@ def resize_lvol(id, new_size):
         logger.error(f"Pool is disabled")
         return False
 
+    new_size = utils.parse_size(new_size)
+
     if lvol.size >= new_size:
         logger.error(f"New size {new_size} must be higher than the original size {lvol.size}")
         return False
@@ -1162,27 +1130,51 @@ def resize_lvol(id, new_size):
         logger.error(f"New size {new_size} must be smaller than the max size {lvol.max_size}")
         return False
 
-    if lvol.cloned_from_snap:
-        logger.error(f"Can not resize clone!")
+    if 0 < pool.lvol_max_size < new_size:
+        logger.error(f"Pool Max LVol size is: {utils.humanbytes(pool.lvol_max_size)}, LVol size: {utils.humanbytes(new_size)} must be below this limit")
         return False
 
-    logger.info(f"Resizing LVol: {lvol.get_id()}, new size: {new_size}")
+    if pool.pool_max_size > 0:
+        total = pool_controller.get_pool_total_capacity(pool.get_id())
+        if total + new_size > pool.pool_max_size:
+            logger.error( f"Invalid LVol size: {utils.humanbytes(new_size)} " 
+                          f"Pool max size has reached {utils.humanbytes(total+new_size)} of {utils.humanbytes(pool.pool_max_size)}")
+            return False
 
     snode = db_controller.get_storage_node_by_id(lvol.node_id)
 
+    logger.info(f"Resizing LVol: {lvol.get_id()} on primary node: {snode.get_id()}")
+    logger.info(f"Current size: {utils.humanbytes(lvol.size)}, new size: {utils.humanbytes(new_size)}")
+
     # creating RPCClient instance
     rpc_client = RPCClient(
-        snode.mgmt_ip,
-        snode.rpc_port,
-        snode.rpc_username,
-        snode.rpc_password)
-
+        snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
+    rpc_client.bdev_lvol_set_leader(True, lvs_name=lvol.lvs_name)
     size_in_mib = int(new_size / (1000*1000))
-    ret = rpc_client.bdev_lvol_resize(lvol.top_bdev, size_in_mib)
+    ret = rpc_client.bdev_lvol_resize(f"{lvol.lvs_name}/{lvol.lvol_bdev}", size_in_mib)
     if not ret:
-        logger.error("Error resizing lvol")
+        logger.error(f"Error resizing lvol on node: {snode.get_id()}")
         return False
 
+    if lvol.ha_type == "ha":
+        sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
+        if sec_node and sec_node.status == StorageNode.STATUS_ONLINE:
+            sec_node_rpc_client = RPCClient(
+                sec_node.mgmt_ip, sec_node.rpc_port, sec_node.rpc_username, sec_node.rpc_password)
+            time.sleep(3)
+            logger.info(f"Resizing LVol: {lvol.get_id()} on secondary node: {sec_node.get_id()}")
+            sec_node_rpc_client.bdev_lvol_set_leader(False, lvs_name=lvol.lvs_name)
+            ret = sec_node_rpc_client.bdev_lvol_resize(f"{lvol.lvs_name}/{lvol.lvol_bdev}", size_in_mib)
+            if not ret:
+                logger.error(f"Error resizing lvol on node: {sec_node.get_id()}")
+                logger.info(f"Revert size on node {snode.get_id()}")
+                size_in_mib = int(lvol.size / (1000 * 1000))
+                ret = rpc_client.bdev_lvol_resize(f"{lvol.lvs_name}/{lvol.lvol_bdev}", size_in_mib)
+                if not ret:
+                    logger.error(f"Failed to revert size for lvol on node {snode.get_id()}, LVol current size: {utils.humanbytes(new_size)}")
+                return False
+
+    lvol = db_controller.get_lvol_by_id(id)
     lvol.size = new_size
     lvol.write_to_db(db_controller.kv_store)
     logger.info("Done")
@@ -1245,7 +1237,14 @@ def get_capacity(lvol_uuid, history, records_count=20, parse_sizes=True):
     records_list = db_controller.get_lvol_stats(lvol, limit=records_number)
     if not records_list:
         return False
-    new_records = utils.process_records(records_list, min(records_count, len(records_list)))
+    cap_stats_keys = [
+        "date",
+        "size_total",
+        "size_used",
+        "size_free",
+        "size_util",
+    ]
+    new_records = utils.process_records(records_list, records_count, keys=cap_stats_keys)
 
     if not parse_sizes:
         return new_records
@@ -1262,7 +1261,7 @@ def get_capacity(lvol_uuid, history, records_count=20, parse_sizes=True):
     return out
 
 
-def get_io_stats(lvol_uuid, history, records_count=20, parse_sizes=True):
+def get_io_stats(lvol_uuid, history, records_count=20, parse_sizes=True, with_sizes=False):
     lvol = db_controller.get_lvol_by_id(lvol_uuid)
     if not lvol:
         logger.error(f"LVol not found: {lvol_uuid}")
@@ -1279,7 +1278,43 @@ def get_io_stats(lvol_uuid, history, records_count=20, parse_sizes=True):
     records_list = db_controller.get_lvol_stats(lvol, limit=records_number)
     if not records_list:
         return False
-    new_records = utils.process_records(records_list, min(records_count, len(records_list)))
+    io_stats_keys = [
+        "date",
+        "read_bytes",
+        "read_bytes_ps",
+        "read_io_ps",
+        "read_latency_ps",
+        "write_bytes",
+        "write_bytes_ps",
+        "write_io_ps",
+        "write_latency_ps",
+        "connected_clients",
+    ]
+    if with_sizes:
+        io_stats_keys.extend(
+            [
+                "size_total",
+                "size_prov",
+                "size_used",
+                "size_free",
+                "size_util",
+                "size_prov_util",
+                "read_latency_ticks",
+                "record_duration",
+                "record_end_time",
+                "record_start_time",
+                "unmap_bytes",
+                "unmap_bytes_ps",
+                "unmap_io",
+                "unmap_io_ps",
+                "unmap_latency_ps",
+                "unmap_latency_ticks",
+                "write_bytes_ps",
+                "write_latency_ticks",
+            ]
+        )
+    # combine records
+    new_records = utils.process_records(records_list, records_count, keys=io_stats_keys)
 
     if not parse_sizes:
         return new_records
