@@ -27,7 +27,7 @@ class RPCClient:
     # ref: https://spdk.io/doc/jsonrpc.html
     DEFAULT_ALLOWED_METHODS = ["HEAD", "GET", "PUT", "DELETE", "OPTIONS", "TRACE", "POST"]
 
-    def __init__(self, ip_address, port, username, password, timeout=60, retry=3):
+    def __init__(self, ip_address, port, username, password, timeout=180, retry=3):
         self.ip_address = ip_address
         self.port = port
         self.url = 'http://%s:%s/' % (self.ip_address, self.port)
@@ -138,13 +138,13 @@ class RPCClient:
         params = {
             "trtype": trtype,
             "max_io_qpairs_per_ctrlr": qpair_count,
-            "max_queue_depth": 512,
+            "max_queue_depth": 2048,
             "abort_timeout_sec": 5,
-            "ack_timeout": 512,
+            "ack_timeout": 2048,
             "zcopy": True,
             "in_capsule_data_size": 4096,
             "max_io_size": 131072,
-            "io_unit_size": 131072,
+            "io_unit_size": 8192,
             "max_aq_depth": 128,
             "num_shared_buffers": 8192,
             "buf_cache_size": 32,
@@ -480,7 +480,8 @@ class RPCClient:
             "name": name,
             "raid_level": raid_level,
             "strip_size_kb": strip_size_kb,
-            "base_bdevs": bdevs_list
+            "base_bdevs": bdevs_list,
+            "io_unmap_limit": 100
         }
         if raid_level == "1":
             params["strip_size_kb"] = 0
@@ -594,13 +595,13 @@ class RPCClient:
     def bdev_nvme_set_options(self):
         params = {
             # "action_on_timeout": "abort",
-            "bdev_retry_count": 0,
-            "transport_retry_count": 0,
+            "bdev_retry_count": 3,
+            "transport_retry_count": 5,
             "ctrlr_loss_timeout_sec": 1,
             "fast_io_fail_timeout_sec": 0,
             "reconnect_delay_sec": 1,
             "keep_alive_timeout_ms": 10000,
-            "transport_ack_timeout": 9,
+            "transport_ack_timeout": 10,
             "timeout_us": constants.NVME_TIMEOUT_US
         }
         return self._request("bdev_nvme_set_options", params)
@@ -908,15 +909,17 @@ class RPCClient:
             params = {"jm_vuid": jm_vuid}
         return self._request("bdev_distrib_force_to_non_leader", params)
 
-    def bdev_lvol_set_leader(self, is_leader=False, uuid=None, lvs_name=None):
+    def bdev_lvol_set_leader(self, is_leader=False, uuid=None, lvs_name=None, bs_nonleadership=False):
         params = {
-            "leadership": is_leader,
+            "lvs_leadership": is_leader,
         }
         if uuid:
             params["uuid"] = uuid
         elif lvs_name:
             params["lvs_name"] = lvs_name
 
+        params["bs_nonleadership"] = bs_nonleadership
+
         return self._request("bdev_lvol_set_leader_all", params)
 
     def bdev_lvol_register(self, name, lvs_name, registered_uuid, blobid, priority_class=0):
@@ -975,9 +978,10 @@ class RPCClient:
         }
         return self._request("nvmf_set_max_subsystems", params)
 
-    def bdev_lvol_set_lvs_groupid(self, lvs_name, groupid):
+    def bdev_lvol_set_lvs_ops(self, lvs_name, groupid, subsystem_port=4420):
         params = {
             "groupid": groupid,
             "lvs_name": lvs_name,
+            "subsystem_port": subsystem_port,
         }
-        return self._request("bdev_lvol_set_lvs_groupid", params)
+        return self._request("bdev_lvol_set_lvs_op", params)
