@@ -49,7 +49,7 @@ def add_lvol_stats(cluster, pool, lvol, stats_list, capacity_dict=None, connecte
         size_used = 0
         lvol_dict = capacity_dict[0]
         size_total = int(lvol_dict['num_blocks']*lvol_dict['block_size'])
-        cluster_size = cluster.ndcs * cluster.page_size_in_blocks
+        cluster_size = cluster.distr_ndcs * cluster.page_size_in_blocks
         if "driver_specific" in lvol_dict and "lvol" in lvol_dict["driver_specific"]:
             num_allocated_clusters = lvol_dict["driver_specific"]["lvol"]["num_allocated_clusters"]
             size_used = int(num_allocated_clusters*cluster_size)
