@@ -84,9 +84,9 @@ def get_iface_ip(ifname):
     return False
 
 
-def print_table(data: list):
+def print_table(data: list, title=None):
     if data:
-        x = PrettyTable(field_names=data[0].keys(), max_width=70)
+        x = PrettyTable(field_names=data[0].keys(), max_width=70, title=title)
         x.align = 'l'
         for node_data in data:
             row = []
@@ -143,10 +143,12 @@ def get_docker_client(cluster_id=None):
     return False
 
 
-def dict_agg(data, mean=False):
+def dict_agg(data, mean=False, keys=None):
     out = {}
+    if not keys and data:
+        keys = data[0].keys()
     for d in data:
-        for key in d.keys():
+        for key in keys:
             if isinstance(d[key], int) or isinstance(d[key], float):
                 if key in out:
                     out[key] += d[key]
@@ -249,11 +251,11 @@ def parse_history_param(history_string):
         if ind == 'm':
             history_in_seconds += v * 60
 
-    records_number = int(history_in_seconds/2)
+    records_number = int(history_in_seconds/5)
     return records_number
 
 
-def process_records(records, records_count):
+def process_records(records, records_count, keys=None):
     # combine records
     if not records:
         return []
@@ -267,7 +269,7 @@ def process_records(records, records_count):
         last_index = (i + 1) * data_per_record
         last_index = min(last_index, len(records))
         sl = records[first_index:last_index]
-        rec = dict_agg(sl, mean=True)
+        rec = dict_agg(sl, mean=True, keys=keys)
         new_records.append(rec)
     return new_records
 
@@ -311,6 +313,9 @@ def get_random_vuid():
                 continue
             used_vuids.append(vuid)
 
+    for lvol in db_controller.get_lvols():
+        used_vuids.append(lvol.vuid)
+
     r = 1 + int(random.random() * 10000)
     while r in used_vuids:
         r = 1 + int(random.random() * 10000)
@@ -511,7 +516,7 @@ def calculate_pool_count(alceml_count, number_of_distribs, cpu_count, poller_cou
 
 def calculate_minimum_hp_memory(small_pool_count, large_pool_count, lvol_count, max_prov, cpu_count):
     '''
-    1092 (initial consumption) + 4 * CPU + 1.0277 * POOL_COUNT(Sum in MB) + (7) * lvol_count
+    1092 (initial consumption) + 4 * CPU + 1.0277 * POOL_COUNT(Sum in MB) + (25) * lvol_count
     then you can amend the expected memory need for the creation of lvols (6MB),
     connection number over lvols (7MB per connection), creation of snaps (12MB),
     extra buffer 2GB
@@ -519,13 +524,13 @@ def calculate_minimum_hp_memory(small_pool_count, large_pool_count, lvol_count,
     '''
     pool_consumption = (small_pool_count * 8 + large_pool_count * 128) / 1024 + 1092
     max_prov_tb = max_prov / (1024 * 1024 * 1024 * 1024)
-    memory_consumption = (4 * cpu_count + 1.0277 * pool_consumption + 7 * lvol_count) * (1024 * 1024) + (250 * 1024 * 1024) * 1.1 * max_prov_tb + constants.EXTRA_HUGE_PAGE_MEMORY
+    memory_consumption = (4 * cpu_count + 1.0277 * pool_consumption + 25 * lvol_count) * (1024 * 1024) + (250 * 1024 * 1024) * 1.1 * max_prov_tb + constants.EXTRA_HUGE_PAGE_MEMORY
     return int(memory_consumption)
 
 
-def calculate_minimum_sys_memory(max_prov):
+def calculate_minimum_sys_memory(max_prov, total):
     max_prov_tb = max_prov / (1024 * 1024 * 1024 * 1024)
-    minimum_sys_memory = (250 * 1024 * 1024) * 1.1 * max_prov_tb + constants.EXTRA_SYS_MEMORY
+    minimum_sys_memory = (250 * 1024 * 1024) * 1.1 * max_prov_tb + (constants.EXTRA_SYS_MEMORY * total)
     logger.debug(f"Minimum system memory is {humanbytes(minimum_sys_memory)}")
     return int(minimum_sys_memory)
 
@@ -664,6 +669,9 @@ def strfdelta(tdelta):
 
 def handle_task_result(task: JobSchedule, res: dict, allowed_error_codes = None):
     if res:
+        if not allowed_error_codes:
+            allowed_error_codes = [0]
+
         res_data = res[0]
         migration_status = res_data.get("status")
         error_code = res_data.get("error", -1)
@@ -690,6 +698,14 @@ def handle_task_result(task: JobSchedule, res: dict, allowed_error_codes = None)
             task.write_to_db()
             return True
 
+        elif migration_status == "none":
+            task.function_result = f"mig retry after restart"
+            task.retry += 1
+            task.status = JobSchedule.STATUS_SUSPENDED
+            del task.function_params['migration']
+            task.write_to_db()
+            return True
+
         else:
             task.function_result = f"Status: {migration_status}, progress:{progress}"
             task.write_to_db()
